<?php
/**
 * This file is part of Rapid Cache
 *
 * @license GPLv3 (See LICENSE.txt for more details)
 * @copyright 2020 MegaOptim (https://megaoptim.com)
 * @copyright 2016 WP Sharks (https://wpsharks.com/)
 */
namespace MegaOptim\RapidCache\Classes;

use MegaOptim\RapidCache\Traits;

/**
 * Notes.
 *
 * @since 1.0.0
 */
class Notes extends AbsBase
{
    use Traits\Shared\StringUtils;

    /**
     * @since 1.0.0
     *
     * @var array
     */
    protected $notes;

    /**
     * Class constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        parent::__construct();

        $this->notes = [];
    }

    /**
     * Get notes.
     *
     * @since 1.0.0
     *
     * @return array Notes.
     */
    public function notes()
    {
        return $this->notes;
    }

    /**
     * Add note.
     *
     * @param string $key  Key.
     * @param string $note Note.
     *
     * @since 1.0.0
     */
    public function add($key, $note)
    {
        $this->notes[$key] = $note;
    }

    /**
     * Add line break.
     *
     * @since 1.0.0
     */
    public function addLineBreak()
    {
        $this->notes[] = "\n";
    }

    /**
     * Add divider.
     *
     * @since 1.0.0
     */
    public function addDivider()
    {
        $this->notes[] = str_repeat('.', 70);
    }

    /**
     * Add ASCII artwork.
     *
     * @param string $note Note.
     *
     * @since 1.0.0
     */
    public function addAsciiArt($note)
    {
        $this->notes[] = '*´¨)
     ¸.•´¸.•*´¨) ¸.•*¨)
     (¸.•´ (¸.•` ¤ '.$note.' ¤ ´¨)';
    }

    /**
     * As HTML comments.
     *
     * @since 1.0.0
     */
    public function asHtmlComments()
    {
        $html_comments    = ''; // Initialize.
        $longest_key_size = 0; // Initialize.

        foreach ($this->notes as $_key => $_note) {
            if (is_string($_key) && $_key && isset($_note[0])) {
                $longest_key_size = max($longest_key_size, mb_strlen($_key.':'));
            }
        } // unset($_key, $_note); // Housekeeping.

        foreach ($this->notes as $_key => $_note) {
            if (is_integer($_key)) {
                if ($_note === "\n") {
                    $html_comments .= "\n";
                } elseif (isset($_note[0])) {
                    $html_comments .= '<!-- '.htmlspecialchars($_note).' -->'."\n";
                }
            } elseif ($_key && !isset($_note[0])) {
                $html_comments .= '<!-- '.htmlspecialchars($_key).' -->'."\n";
            } elseif (!$_key && isset($_note[0])) {
                $html_comments .= '<!-- '.htmlspecialchars($_note).' -->'."\n";
            } elseif ($_key && isset($_note[0])) {
                $html_comments .= '<!-- '.htmlspecialchars($this->strPad($_key.':', $longest_key_size).'    '.$_note).' -->'."\n";
            }
        } // unset($_key, $_note); // Housekeeping.

        return trim($html_comments);
    }
}
